using System;

namespace Cerebrum.Ganglion.Ensemble
{
	internal interface IDatabaseController : Cerebrum.Ganglion.Remoting.IDatabaseController
	{
		Cerebrum.IWorkspace Workspace {get;}
	}
	/// <summary>
	/// Summary description for DatabaseController.
	/// </summary>
	public class DatabaseController : System.MarshalByRefObject, IDatabaseController, System.IDisposable
	{
		private string m_DomainName;
		private Cerebrum.Integrator.IContextService m_ContextService;
		private Cerebrum.IWorkspace m_Workspace;
		internal DatabaseController(string name, Cerebrum.IConnector domainDescriptor)
		{
			Cerebrum.Integrator.DomainContext MasterContext = Cerebrum.Ganglion.Ensemble.ControllerImplementation.Instance.Application.MasterContext;
			Cerebrum.Ganglion.Ensemble.Reflection.DomainDescriptor dd = domainDescriptor.Component as Cerebrum.Ganglion.Ensemble.Reflection.DomainDescriptor;

			string primaryDirectory = MasterContext!=null?(MasterContext.SystemDirectory):System.IO.Path.GetDirectoryName(this.GetType().Assembly.Location);
			string databaseFileName = System.IO.Path.Combine(primaryDirectory, dd.DatabaseFileName);
			string activityFileName = System.IO.Path.Combine(primaryDirectory, dd.ActivityFileName);
			string portableFileName = System.IO.Path.Combine(primaryDirectory, dd.PortableFileName);
			m_DomainName = dd.Name;

			m_ContextService = new Cerebrum.Management.SimpleContextService(ControllerImplementation.Instance.Application, databaseFileName, activityFileName, 8, true, portableFileName, true);
			m_Workspace = null;
		}

		public void Initialize()
		{
			if(m_Workspace==null)
			{
				lock(ControllerImplementation.SyncRoot)
				{
					if(m_Workspace==null)
					{
						Cerebrum.Ganglion.Ensemble.Application application = ControllerImplementation.Instance.Application;

						
						string databaseFileName = m_ContextService.DatabaseFileName;

						bool isNew; 
						if(System.IO.File.Exists(databaseFileName))
						{
							m_Workspace = m_ContextService.LoadContext(databaseFileName);
							isNew = false;
						}
						else
						{
							m_Workspace = m_ContextService.InitContext(databaseFileName, this.m_DomainName);
							isNew = true;
						}
						Cerebrum.Integrator.DomainContext c = m_Workspace.Component as Cerebrum.Integrator.DomainContext;

						m_ContextService.ImportDatabase(c, isNew);

						application.AddDomain(c);

						c.Initialize();
					}
				}
			}
		}

		public void Shutdown()
		{
			if(m_Workspace!=null)
			{
				lock(ControllerImplementation.SyncRoot)
				{
					if(m_Workspace!=null)
					{
						Cerebrum.Integrator.DomainContext domainContext = Cerebrum.Integrator.DomainContext.FromConnector(m_Workspace as Cerebrum.IConnector);
						domainContext.Workspace.Activator = null;
						domainContext.Shutdown();
						m_Workspace = null;
					}
				}
			}
		}
		public Cerebrum.IWorkspace Workspace 
		{
			get
			{
				return new SyncronizedWorkspace(m_Workspace);
			}
		}

		internal Cerebrum.Integrator.DomainContext DomainContext
		{
			get
			{
				this.Initialize();
				return Cerebrum.Integrator.DomainContext.FromConnector(this.m_Workspace);
			}
		}

		public Cerebrum.ObjectHandle InsertRecord(string tableName, string []columns, object []values)
		{
			lock(ControllerImplementation.SyncRoot)
			{
				Cerebrum.Data.ComponentItemView o = null;
				Cerebrum.Integrator.DomainContext domain = this.DomainContext;
				using(Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(domain, tableName))
				{
					System.ComponentModel.PropertyDescriptorCollection pds = (view as System.ComponentModel.ITypedList).GetItemProperties(null);
					o = view.AddNew();
					for(int i=0; i< columns.Length; i++)
					{
						string column = columns[i];
						System.ComponentModel.PropertyDescriptor pd = pds[column];

						pd.SetValue(o, values[i]);
					}
				}
				Cerebrum.ObjectHandle h = o.ObjectHandle;
				this.Shutdown();
				return h;
			}
		}

		public bool UpdateRecord(string tableName, Cerebrum.ObjectHandle handle, string []columns, object []values)
		{
			lock(ControllerImplementation.SyncRoot)
			{
				Cerebrum.Integrator.DomainContext domain = this.DomainContext;
				using(Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(domain, tableName))
				{
					System.ComponentModel.PropertyDescriptorCollection pds = (view as System.ComponentModel.ITypedList).GetItemProperties(null);
					object o = view.CreateItemViewByObjectHandle(new Cerebrum.ObjectHandle(handle.ToIntPtr()));
					for(int i=0; i< columns.Length; i++)
					{
						string column = columns[i];
						System.ComponentModel.PropertyDescriptor pd = pds[column];

						pd.SetValue(o, values[i]);
					}
				}
				this.Shutdown();
				return true;
			}
		}
		public bool EngageRecord(string tableName, Cerebrum.ObjectHandle handle)
		{
			lock (ControllerImplementation.SyncRoot)
			{
				Cerebrum.Integrator.DomainContext domain = this.DomainContext;
				using (Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(domain, tableName))
				{
					using (Cerebrum.IComposite connector = domain.AttachConnector(handle))
					{
						view.Add(handle, connector);
					}
				}
				this.Shutdown();
				return true;
			}
		}
		public object InvokeRecord(string tableName, Cerebrum.ObjectHandle handle, string method, object[] parameters)
		{
			lock (ControllerImplementation.SyncRoot)
			{
				Cerebrum.Integrator.DomainContext domain = this.DomainContext;
				object v = Tools.InvokeRecord(domain, tableName, handle, method, parameters);
				this.Shutdown();
				return v;
			}
		}
		public bool DeleteRecord(string tableName, Cerebrum.ObjectHandle handle)
		{
			lock(ControllerImplementation.SyncRoot)
			{
				Cerebrum.Integrator.DomainContext domain = this.DomainContext;
				using(Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(domain, tableName))
				{
					view.Remove(view.CreateItemViewByObjectHandle(handle));
					domain.Workspace.RemoveConnector(handle);
				}
				this.Shutdown();
				return true;
			}
		}

		public object [] SelectRecord(string tableName, Cerebrum.ObjectHandle handle, string []columns)
		{
			lock(ControllerImplementation.SyncRoot)
			{
				object [] values = new object[columns.Length];
				Cerebrum.Integrator.DomainContext domain = this.DomainContext;
				using(Cerebrum.Data.TableView view = Cerebrum.Management.Utilites.GetView(domain, tableName))
				{
					System.ComponentModel.PropertyDescriptorCollection pds = (view as System.ComponentModel.ITypedList).GetItemProperties(null);
					object o = view.CreateItemViewByObjectHandle(handle);
					for(int i=0; i< columns.Length; i++)
					{
						string column = columns[i];
						System.ComponentModel.PropertyDescriptor pd = pds[column];

						values[i] = pd.GetValue(o);
					}
				}
				return values;
			}
		}
		#region IDisposable Members

		public void Dispose()
		{
			this.Shutdown();
			m_ContextService = null;
		}

		#endregion

		#region IDatabaseController Members


		public ObjectHandle ResolveHandle(string tableName, string fieldName, object value)
		{
			lock (ControllerImplementation.SyncRoot)
			{
				Cerebrum.Integrator.DomainContext domain = this.DomainContext;
				return Cerebrum.Management.Utilites.ResolveHandle(domain, tableName, fieldName, value);
			}
		}

		#endregion
	}
}
